@@ -3,5 +3,5 @@
   cat("-----------------------------\n")
   cat("You can ignore comments from pbatR above to use 'fbati()' and 'fbatj()'.\n")
   #cat("\nPackage fbati: type 'fbati()' or 'fbatj()' to open up the gui interface; the command line interface is the same function but with options specified.\n" )
-  cat("Enter\n launchpad()\nto start up the menuing interface. Users may prefer the 'fbati()' and 'fbatj()' functions.\n")
+  cat("Enter\n launchpad()\nto start up the menuing interface. Users may prefer the 'fbati()', 'fbatj()', and 'fbatc()' functions.\n")
 }
